'''
PyMOL Surflex xGen GUI Plugin

(c) BioPharmics LLC
'''

from __future__ import absolute_import
from __future__ import print_function

import sys
import os
import importlib

from pymol.Qt import QtCore, QtWidgets
from pymol.Qt.utils import loadUi, AsyncFunc, MainThreadCaller
from PyQt5.QtWidgets import QTableWidget, QTableWidgetItem

getOpenFileNames = QtWidgets.QFileDialog.getOpenFileNames
getExistingDirectory = QtWidgets.QFileDialog.getExistingDirectory


##################################################################################
#  Local python files:
#      sffuncs contains the actual code to fork processes
#      sfglob contains the globals that pass interface parameters to sffuncs
##################################################################################
from . import sffuncs
from . import sfglob


class QMessageBox(QtWidgets.QMessageBox):

    _EVENT_TYPES = (QtCore.QEvent.UpdateRequest, QtCore.QEvent.WinIdChange, QtCore.QEvent.ShowToParent, )
    _UNWANTED_WINDOW_FLAGS = ( QtCore.Qt.MSWindowsFixedSizeDialogHint | 0)

    def _make_resizable(self):
        textEdit = self.findChild(QtWidgets.QTextEdit)
        if textEdit is None:
            return

        self.setSizeGripEnabled(True)

        ex = QtWidgets.QSizePolicy.Expanding
        for w in [self, textEdit]:
            w.setMaximumSize(0xffffff, 0xffffff)
            w.setSizePolicy(ex, ex)

    def event(self, e):
        if e.type() in self._EVENT_TYPES:
            self._make_resizable()

        return super(QMessageBox, self).event(e)


class SilentAbort(Exception):
    pass


class StdOutCapture:
    '''
Redirect stdout and/or stderr to a temporary file until 'release' is called.
    '''

    def __init__(self, stdout=True, stderr=False):
        import tempfile
        self.temp = tempfile.TemporaryFile('w+')
        self.save_stdout = None
        self.save_stderr = None
        if stdout:
            self.save_stdout = sys.stdout
            sys.stdout = self.temp
        if stderr:
            self.save_stderr = sys.stderr
            sys.stderr = self.temp

    def release(self):
        if self.save_stdout is not None:
            sys.stdout = self.save_stdout
        if self.save_stderr is not None:
            sys.stderr = self.save_stderr
        self.temp.seek(0)
        content = self.temp.read()
        self.temp.close()
        return content


def surflex_register():
    import webbrowser
    webbrowser.open("https://www.biopharmics.com/downloads/")

def write_xgen_file(form,xgen_step):
    save_dir = form.xgen_outdir.text()
    xgen_filename = form.xgen_def_file.text()

    rpath = os.path.join(save_dir, xgen_filename)
    rfile = open(rpath,'w')
    rfile.write('xgen_dir' + ' ' + form.xgen_outdir.text() + '\n')
    rfile.write('xgen_protein' + ' ' + form.xgen_protein.text() + '\n')
    rfile.write('map' + ' ' + form.xgen_map.text() + '\n')
    rfile.write('resolution' + ' ' + format(form.xgen_res.value(),".3f") + '\n')
    rfile.write('carve' + ' ' + format(form.xgen_dcarve.value(),".1f") + '\n')
    rfile.write('dbox_scope_lig' + ' ' + form.xgen_dbox_scope.text() + '\n')
    rfile.write('dbox_outprefix' + ' ' + form.xgen_outprefix.text() + '\n')
    if xgen_step == 'dbox':
        rfile.close()
        return

    rfile.write('build_carve' + ' ' + format(form.xgen_build_carve.value(),".3f") + '\n')
    rfile.write('build_outprefix' + ' ' + form.xgen_build_outprefix.text() + '\n')
    if xgen_step == 'build':
        rfile.close()
        return

    rfile.write('sf_outprefix' + ' ' + form.xgen_build_outprefix.text() + '\n')
    rfile.write('xgref_lig' + ' ' + form.xgen_refine_lig.text() + '\n')
    rfile.write('xgref_density' + ' ' + form.xgen_constraints.text() + '\n')
    if xgen_step == 'refine':
        rfile.close()
        return

    rfile.write('xgens_input' + ' ' + form.xgens_input.text() + '\n')
    rfile.write('xgens_outprefix' + ' ' + form.xgens_outprefix.text() + '\n')
    if xgen_step == 'ensemble':
        rfile.close()
        return

    # Here we have the eval info...
    if xgen_step != 'eval':
        rfile.close()
        return
    
    rfile.write('pdb' + ' ' + form.xgen_pdb.text() + '\n')

    if len(form.RSCC_E.text()) > 0:
        rfile.write('xgens_rscc' + ' ' + form.RSCC_E.text() + '\n')
        rfile.write('xgens_rsr' + ' ' + form.RSR_E.text() + '\n')
        rfile.write('xgorig_rscc' + ' ' + form.RSCC_O.text() + '\n')
        rfile.write('xgorig_rsr' + ' ' + form.RSR_O.text() + '\n')

    rfile.close()
    

def homogenize_file(form, infile):
    xgen_dir = os.path.realpath(form.xgen_outdir.text())
    head, tail = os.path.split(infile)
    in_dir = os.path.realpath(head)

    # If we are not making local copies, we keep the whole file name
    if form.check_no_copy.isChecked() == 1:
        return infile

    if in_dir == xgen_dir:
        return tail
    else:
        to_location = os.path.join(xgen_dir,tail)
        shutil.copyfile(infile,to_location)
        return tail

def clear_xgen_ensemble(form):
    from pymol import cmd
    
    form.xgens_input.setText('')
    form.xgen_table.setRowCount(0)
    if sffuncs.sele_exists('Ensemble'): cmd.delete('Ensemble')
    if sffuncs.sele_exists('Ensemble-Iso10'): cmd.delete('Ensemble-Iso10')

def clear_xgen_eval(form):
    from pymol import cmd

    form.RSCC_E.setText('')
    form.RSCC_O.setText('')
    form.RSR_E.setText('')
    form.RSR_O.setText('')
    if sffuncs.sele_exists('XGEval_Contour1.0'): cmd.delete('XGEval_Contour1.0')
    if sffuncs.sele_exists('OrigEval_Contour1.0'): cmd.delete('OrigEval_Contour1.0')

def update_xgen_eval_stats(form):
    # Now, we will populate the ensemble information table
    epath = os.path.join(form.xgen_outdir.text(), form.xgens_outprefix.text() + '-eval-log')
        
    efile = open(epath,'r')
    for line in efile:
        if len(line) > 1:
            xgen_args = line.split()
            token = xgen_args[0]
            if token == 'RSCC':
                value = xgen_args[2]
                form.RSCC_E.setText(value)
            if token == 'RSR':
                value = xgen_args[2]
                form.RSR_E.setText(value)
    efile.close()

    epath = os.path.join(form.xgen_outdir.text(), form.xgens_outprefix.text() + '-pdbeval-log')
    efile = open(epath,'r')
    for line in efile:
        if len(line) > 1:
            xgen_args = line.split()
            token = xgen_args[0]
            if token == 'RSCC':
                value = xgen_args[2]
                form.RSCC_O.setText(value)
            if token == 'RSR':
                value = xgen_args[2]
                form.RSR_O.setText(value)
    efile.close()

def update_xgen_ensemble_stats(form):
    # Now, we will populate the ensemble information table
    trio_prefix = form.sf_outprefix.text()
    trio_log = trio_prefix + '-trios-log'

    # Grab the best energy and density
    f = open(trio_log,'r')
    for line in f:
        if len(line) > 1:
            line_args = line.split()
            token = line_args[0]
            if token == 'Best':
                best_energy = line_args[2]
                best_density = line_args[5]
            if len(line_args) == 11:
                token = line_args[8]
                if token == 'n_neighborhoods':
                    n_trios = int(line_args[10])
    f.close()

    # Let's write some stuff in the xgen_table
    form.xg_best_energy.setText(best_energy)
    form.xg_best_density.setText(best_density)

    n_confs = 0
    # Resize the table
    ensemble_prefix = form.xgens_outprefix.text()
    ensemble_log = ensemble_prefix + '-log'
    f = open(ensemble_log,'r')
    for line in f:
        if len(line) > 1:
            line_args = line.split()
            if len(line_args) == 3:
                token = line_args[0]
                if token == 'NFINAL':
                    n_confs = int(line_args[2])
    f.close()
    form.xgen_table.setRowCount(n_confs)
    
    # Grab the occupancies
    f = open(ensemble_log,'r')
    for line in f:
        if len(line) > 1:
            line_args = line.split()
            if len(line_args) == 8:
                token = line_args[5]
                if token == 'OCCUPANCY':
                    idx = int(line_args[1])-1
                    occ = line_args[7]
                    item = QTableWidgetItem(occ)
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    form.xgen_table.setItem(idx, 0, item)
                    # Now we need to grab the trio property values, which
                    # will require the the indices from the cn_005 sorts
                    # of things in the ensemble log.
                    # From those, we can figure out which trio in the xgenout-trios-log file
                    cnum_txt = line_args[3]
                    cnum_args = cnum_txt.split("_")
                    cnum = int(cnum_args[1])
                    trio_num = cnum % n_trios
                    if trio_num == 0: trio_num = n_trios
                    trio_type = cnum / n_trios
                    trio_string = 'cn_' + format(trio_num,"03d")

                    tenergy = '---'
                    tdfit = '---'
                    tf = open(trio_log,'r')
                    for tline in tf:
                        tline_args = tline.split()
                        if len(tline_args) == 15:
                            if tline_args[0] == trio_string:
                                tenergy = ('+' + format(float(tline_args[13])-float(best_energy),".2f"))
                                tdfittxt = tline_args[4]
                                tdfit = format(float(tdfittxt[1:len(tdfittxt)])/float(best_density),".3f")
                    tf.close()

                    item = QTableWidgetItem(tenergy)
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    form.xgen_table.setItem(idx, 1, item)

                    item = QTableWidgetItem(tdfit)
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    form.xgen_table.setItem(idx, 2, item)

    f.close()



def read_xgenfile(filepath,form):
    from pymol import cmd

    have_eval = 0
    have_build = 0
    have_ensemble = 0
    
    f = open(filepath,'r')
    for line in f:
        if len(line) > 1:
            xgen_args = line.split()
            if len(xgen_args) == 2:
                token = xgen_args[0]
                value = xgen_args[1]
                if token == 'xgen_dir':
                    form.xgen_outdir.setText(value)                
                if token == 'pdb':
                    form.xgen_pdb.setText(value)
                if token == 'map':
                    form.xgen_map.setText(value)
                if token == 'resolution':
                    form.xgen_res.setValue(float(value))
                if token == 'carve':
                    form.xgen_dcarve.setValue(float(value))
                if token == 'dbox_scope_lig':
                    form.xgen_dbox_scope.setText(value)
                    form.xgen_refine_lig.setText(value)
                if token == 'dbox_outprefix':
                    form.xgen_outprefix.setText(value)
                if token == 'build_carve':
                    form.xgen_build_carve.setValue(float(value))
                if token == 'build_outprefix':
                    have_build = 1
                    form.xgen_build_outprefix.setText(value)
                if token == 'xgref_lig':
                    form.xgen_refine_lig.setText(value)
                if token == 'xgref_density':
                    form.xgen_constraints.setText(value)
                if token == 'xgens_input':
                    have_ensemble = 1
                    form.xgens_input.setText(value)
                if token == 'xgen_protein':
                    form.xgen_protein.setText(value)
                if token == 'xgens_outprefix':
                    form.xgens_outprefix.setText(value)
                if token == 'xgens_rscc':
                    have_eval = 1
                    form.RSCC_E.setText(value)
                if token == 'xgorig_rscc':
                    form.RSCC_O.setText(value)
                if token == 'xgens_rsr':
                    form.RSR_E.setText(value)
                if token == 'xgorig_rsr':
                    form.RSR_O.setText(value)
    f.close()

    # Check to make sure that the xgen file is in the directory specified as the xgen_dir
    head_xgen, tail_xgen = os.path.split(filepath)
    
    xgen_dir = form.xgen_outdir.text()                
    if os.path.realpath(xgen_dir) != os.path.realpath(head_xgen):
        if QMessageBox.Yes == QMessageBox.question(
                form._dialog, 'Directory Mismatch', 'xGen file and directory do not match. Auto-fix (no --> remove xGen file)?',
                QMessageBox.Yes, QMessageBox.No):
            xgen_dir = os.path.realpath(head_xgen)
            form.xgen_outdir.setText(value)
        else:
            os.remove(filepath)
            return

    # Assume we have nothing
    form.tabWidget.setCurrentIndex(1)

    # Now we will maybe load the things that exist
    in_dir = form.xgen_outdir.text()
    in_file = form.xgen_protein.text()
    loadfile = os.path.join(in_dir, in_file)
    sffuncs.maybe_load_sticks(loadfile,'xg_protein','0.1','0')
    sffuncs.color_carbons('gray40','xg_protein')

    in_dir = form.xgen_outdir.text()
    in_file = form.xgen_protein.text()
    loadfile = os.path.join(in_dir, in_file)
    sffuncs.maybe_load_sticks(loadfile,'xg_protein','0.1','0')
    sffuncs.color_carbons('gray40','xg_protein')

    in_file = form.xgen_dbox_scope.text()
    loadfile = os.path.join(in_dir, in_file)
    sffuncs.maybe_load_sticks(loadfile,'xg_scope','0.11','-1')
    sffuncs.color_carbons('cyan','xg_scope')
    
    in_file = form.xgen_refine_lig.text()
    loadfile = os.path.join(in_dir, in_file)
    sffuncs.maybe_load_sticks(loadfile,'xg_lig','0.20','0')
    sffuncs.color_carbons('green','xg_lig')

    in_file = (form.xgen_outprefix.text() + '_contour.mol2')
    loadfile = os.path.join(in_dir, in_file)
    sffuncs.maybe_load_sticks(loadfile,'Raw_Contour','0.03','-1')
    cmd.spectrum('pc', 'rainbow', 'Raw_Contour')
    
    cmd.hide('everything','(h. and (e. c extend 1))')

    if have_build: 
        # We have finished the idealized approximation
        form.tabWidget.setCurrentIndex(0)

        if sffuncs.sele_exists('Raw_Contour'): cmd.disable('Raw_Contour')
        if sffuncs.sele_exists('map'): cmd.disable('map')

        loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-fit00-iso.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ideal00_Contour','0.03','0')
        loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-fit01-iso.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ideal01_Contour','0.03','0')
        loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-fit02-iso.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ideal02_Contour','0.03','0')
        loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-fit03-iso.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ideal03_Contour','0.03','0')
        loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-fit04-iso.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ideal04_Contour','0.03','0')
        if sffuncs.sele_exists('Ideal00_Contour'):
            cmd.group('Ideal_Contours', 'Ideal00_Contour Ideal01_Contour Ideal02_Contour Ideal03_Contour Ideal04_Contour')
            cmd.show_as('sticks','Ideal_Contours')
            cmd.set('stick_radius', '0.03', 'Ideal_Contours')
            cmd.spectrum('pc', 'rainbow', 'Ideal_Contours')

            loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-exptl00-iso.mol2'))
            sffuncs.maybe_load_sticks(loadfile,'Exptl00_Contour','0.03','0')
            loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-exptl01-iso.mol2'))
            sffuncs.maybe_load_sticks(loadfile,'Exptl01_Contour','0.03','0')
            loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-exptl02-iso.mol2'))
            sffuncs.maybe_load_sticks(loadfile,'Exptl02_Contour','0.03','0')
            loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-exptl03-iso.mol2'))
            sffuncs.maybe_load_sticks(loadfile,'Exptl03_Contour','0.03','0')
            loadfile = os.path.join(in_dir, (form.xgen_build_outprefix.text() + '-exptl04-iso.mol2'))
            sffuncs.maybe_load_sticks(loadfile,'Exptl04_Contour','0.03','0')
            if sffuncs.sele_exists('Exptl00_Contour'):
                cmd.group('Exptl_Contours', 'Exptl00_Contour Exptl01_Contour Exptl02_Contour Exptl03_Contour Exptl04_Contour')
                cmd.show_as('sticks','Exptl_Contours')
                cmd.set('stick_radius', '0.03', 'Exptl_Contours')
                cmd.spectrum('pc', 'rainbow', 'Exptl_Contours')
                cmd.disable('Exptl_Contours')

                if sffuncs.sele_exists('xg_scope'):
                    cmd.zoom('xg_scope')

                    if sffuncs.sele_exists('xg_lig'):
                        cmd.zoom('xg_lig')
                

    if have_ensemble:
        loadfile = os.path.join(in_dir, (form.xgens_outprefix.text() + '-confs.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ensemble','0.10','-1')
        sffuncs.color_carbons('brightorange','Ensemble')
        loadfile = os.path.join(in_dir, (form.xgens_outprefix.text() + '-iso.mol2'))
        sffuncs.maybe_load_sticks(loadfile,'Ensemble-Iso10','0.03','-1')
        sffuncs.color_all('brightorange','Ensemble-Iso10')

        sffuncs.color_carbons('gray40','xg_protein')

        if sffuncs.sele_exists('Ensemble'):
            cmd.disable()
            cmd.enable('Ensemble')
            cmd.enable('Ensemble-Iso10')
            cmd.enable('xg_protein')
            update_xgen_ensemble_stats(form)

            cmd.hide('everything','(h. and (e. c extend 1))')

    # Check if we have xgens_rscc actual value
    # If so, we will load the cool eval stuff
    rscc_string = form.RSCC_E.text()
    if have_eval:
        protein_pdb = form.xgen_pdb.text()
        refine_ligand = form.xgen_refine_lig.text()

        # Load the pdb protein map
        cmd.disable()
        cmd.load(protein_pdb, 'Protein_BFactors', quiet=1)    
        cmd.spectrum('b', 'rainbow', 'Protein_BFactors')
        cmd.set('stick_radius', '0.10', 'Protein_BFactors')
        sffuncs.maybe_load_sticks(refine_ligand,'Orig_Lig','0.2','-1')
        sffuncs.color_carbons('cyan','Orig_Lig')
        cmd.zoom('Orig_Lig')
        cmd.enable('Ensemble')
        
        if not sffuncs.sele_exists('map'):
            save_dir = form.xgen_outdir.text()
            mapfile = form.xgen_map.text()
            cmd.load(mapfile, 'phases', quiet=1)
            cmd.isomesh('map','phases.2fofc',level=1.0,selection='Orig_Lig',state=1,carve=1.6)
            cmd.hide('everything','(h. and (e. c extend 1))')
            cmd.set('mesh_color','gray60')
            cmd.zoom('Orig_Lig')
            cmd.disable('Orig_Lig')

            ensemble_prefix = form.xgens_outprefix.text()            
            eval_prefix = (ensemble_prefix + '-eval')
            pdbeval_prefix = (ensemble_prefix + '-pdbeval')

            # Now load the eval iso
            contour_path = os.path.join(save_dir, (eval_prefix + '-iso10.mol2'))
            sffuncs.maybe_load_sticks(contour_path,'XGEval_Contour1.0','0.03','0')
            sffuncs.color_all('brightorange','XGEval_Contour1.0')
            
            pdbeval_prefix = (ensemble_prefix + '-pdbeval')
            ptile_string = (eval_prefix + '-ptile')
            
            # Now load the pdbeval iso
            contour_path = os.path.join(save_dir, (pdbeval_prefix + '-iso10.mol2'))
            sffuncs.maybe_load_sticks(contour_path,'OrigEval_Contour1.0','0.03','0')
            sffuncs.color_all('cyan','OrigEval_Contour1.0')
            cmd.disable('OrigEval_Contour1.0')

#################################################################################################
# This gets together all information for running anything in the GUI
def run_sfgeneral(form, _self):
    '''
Set up and execute whatever Surflex function is needed
    '''

    # First grab *all* of the info from the GUI boxes and so forth
    # The values will be stuffed into the sfglob globals
    # They will be grabbed within sffuncs by each respective function
    sfglob.mode_name = form.prep_method.currentText()
    sfglob.sf_outprefix = form.sf_outprefix.text()
    sfglob.xg_lig = form.xgen_refine_lig.text()
    sfglob.xg_string = form.xgen_constraints.text()
    sfglob.save_dir = form.xgen_outdir.text()
    sfglob.options = form.surflex_args.text()
    sfglob.tools_exe = form.surflex_exe.text()
    sfglob.check_preserve = form.check_preserve.isChecked()
    sfglob.check_copy = form.check_no_copy.isChecked()
    sfglob.xgen_protein = form.xgen_protein.text()
    sfglob.dbox_map = form.xgen_map.text()
    sfglob.dbox_scope = form.xgen_dbox_scope.text()
    sfglob.xgen_res = form.xgen_res.value()
    sfglob.xgen_dbox_carve = form.xgen_dcarve.value()
    sfglob.xgen_outprefix = form.xgen_outprefix.text()
    sfglob.dprefix = form.xgen_outprefix.text()
    sfglob.build_outprefix = form.xgen_build_outprefix.text()
    sfglob.xgen_build_carve = form.xgen_build_carve.value()
    sfglob.xg_build_contour = form.xgen_build_contour.isChecked()
    sfglob.xgens_outprefix = form.xgens_outprefix.text()
    sfglob.xgens_contour = form.xgens_contour.isChecked()
    sfglob.xgens_diverse = form.xgens_diverse.isChecked()
    sfglob.xgen_constraints = form.xgen_constraints.text()
    sfglob.input_pool = form.xgens_input.text()
    sfglob.xgens_mol2 = form.xgens_outprefix.text() + '-confs.mol2'
    sfglob.xgens_occ = form.xgens_outprefix.text() + '-occ'
    sfglob.xgen_protein_pdb = form.xgen_pdb.text()
    sfglob.origeval_mol2 = form.orig_eval.text()
    sfglob.origeval_occ = form.orig_eval_occ.text()
    sfglob.mapfile = form.xgen_map.text()
    # Here we check if we want to just look at the selected ensemble confs
    if form.eval_selected.isChecked():
        sfglob.xgens_mol2 = 'SelectedEnsemble-confs.mol2'
        sfglob.xgens_occ = 'SelectedEnsemble-occ'

    print('Inside init.run_sfgeneral: dprefix = ', sfglob.command, sfglob.dprefix, file=sys.stderr )
    
    # Here, we will also deal with clearing/modifying any PyMol window-related items if need be
    # NOTE: if GUI-items need to be processed, this should happen in r_sfgeneral_pre (before set_running_abort())
    # Then, call the specific function in sffuncs, which will grab params from sfglob
    if sfglob.command == 'dbox':
        sffuncs.run_build_dbox(_proclist=form._proclist)

    if sfglob.command == 'build':
        sffuncs.run_build_xgbuild(_proclist=form._proclist)

    if sfglob.command == 'refine':
        sffuncs.run_xgen_refine(_proclist=form._proclist)

    if sfglob.command == 'ensemble':
        sffuncs.run_xgensemble(_proclist=form._proclist)

    if sfglob.command == 'eval':
        sffuncs.run_xgeval(_proclist=form._proclist)

    # The function r_sfgeneral_post will handle any post-op work

#################################################################################################

#################################################################################################

def dialog(_self=None):
    if _self is None:
        from pymol import cmd as _self

    dialog = QtWidgets.QDialog()
    uifile = os.path.join(os.path.dirname(__file__), 'surflex.ui')
    form = loadUi(uifile, dialog)
    form._dialog = dialog
    form._proclist = []

    form.xgen_outdir.setText(os.getcwd())

    filename = os.path.join(os.getcwd(), 'refine.xgen')
    if os.path.exists(filename):
        read_xgenfile(filename,form)
    else:
        from pymol import cmd
        my_dir = getExistingDirectory(None,
                                      "Select the xGen project folder",
                                      form.xgen_outdir.text(),
                                      QtWidgets.QFileDialog.ShowDirsOnly)
        form.xgen_outdir.setText(os.path.realpath(my_dir))
        cmd.cd(dir=os.path.realpath(my_dir))
        form.tabWidget.setCurrentIndex(1)
        filename = os.path.join(os.getcwd(), 'refine.xgen')
        if os.path.exists(filename):
            read_xgenfile(filename,form)
    
    # executables
    from distutils.spawn import find_executable
    form.surflex_exe.setText(sffuncs.find_surflex_exe() or 'surflex')
    form.surflex_exe.setEnabled(False)

    # for async panels
    form._callInMainThread = MainThreadCaller()

    ################################################################
    # NOTE: run_sfgeneral is defined above. It gathers all of the
    # form input information, sets globals in sffuncs.py and then
    # executes the run_sfgeneral function
    ################################################################
    run_sfgeneral_async = AsyncFunc(run_sfgeneral)

    def r_sfgeneral_pre():
        form._capture = StdOutCapture()

        # Make GUI changes before locking the dialog box
        if sfglob.command == 'dbox':
            clear_xgen_ensemble(form)
            clear_xgen_eval(form)

        if sfglob.command == 'build':
            clear_xgen_ensemble(form)
            clear_xgen_eval(form)

        if sfglob.command == 'refine':
            clear_xgen_ensemble(form)
            clear_xgen_eval(form)

        if sfglob.command == 'ensemble':
            clear_xgen_eval(form)

        if sfglob.command == 'eval':
            clear_xgen_eval(form)
            # Really need the PDB file now!
            if len(form.xgen_pdb.text()) < 3:
                QMessageBox.about(form._dialog, "PDB Protein", "Need PDB protein file for eval")
                fnames = getOpenFileNames(None, filter='Protein (*.pdb);;All Files (*)')[0]
                if fnames:
                    form.xgen_pdb.setText(homogenize_file(form,fnames[0]))

        # Prevent any other dialog commands from running
        set_running_abort()

        # Run the function asynchronously
        run_sfgeneral_async(form, _self)

    # When the async function is done, we process the following
    @run_sfgeneral_async.finished.connect
    def r_sfgeneral_post(args):
        _, exception = args

        form._proclist[:] = []
        stdout = form._capture.release()
        print(stdout)

        set_running_done()

        # Put state changes for GUI here
        print('In r_sfgeneral_post: ',sfglob.command)

        # Only run this post-stuff if we did not abort
        if sfglob.abort_status == False:
            if sfglob.command == 'dbox':
                write_xgen_file(form,'dbox')

            if sfglob.command == 'build':
                default_xgpath = (form.xgen_build_outprefix.text() + '-ligfit')
                form.xgen_constraints.setText(default_xgpath)
                form.tabWidget.setCurrentIndex(0)
                write_xgen_file(form,'build')

            if sfglob.command == 'refine':
                if form.xgens_diverse.isChecked():
                    form.xgens_input.setText((form.sf_outprefix.text() + '-trios.mol2'))
                else:
                    form.xgens_input.setText((form.sf_outprefix.text() + '-densitybiased.mol2'))
                write_xgen_file(form,'refine')

            if sfglob.command == 'ensemble':
                default_xgpath = (form.xgen_build_outprefix.text() + '-ligfit')
                update_xgen_ensemble_stats(form)
                write_xgen_file(form,'ensemble')

            if sfglob.command == 'eval':
                update_xgen_eval_stats(form)
                write_xgen_file(form,"eval")

        # Handle exceptions here
        if exception is not None:
            handle_exception(exception, stdout)
            return

    ################################################################


    ################################################################
    def xgen_select_ensemble():
        import tempfile
        import shutil
        from pymol import cmd

        rows = []
        for idx in form.xgen_table.selectionModel().selectedRows():
            rows.append(idx.row())

        if (len(rows) > 0):
            # Now we need to get a temp directory to dump the selected ensemble members
            tempdir = tempfile.mkdtemp()
            filenames = []
            for rnum in rows:
                dpath = 'ensemble-' + format(rnum,"03d") + '.mol2'
                dumpfile = os.path.join(tempdir, dpath)
                cmd.save(dumpfile,'Ensemble',rnum+1,'mol2')
                filenames.append(dumpfile)
            # Now we will concat the files into the tmpdir
            newpath = os.path.join(tempdir,'SelEnsemble.mol2')
            sffuncs.concat_files(filenames,newpath)
            save_dir = form.xgen_outdir.text()
            ensemble_file = os.path.join(save_dir, ('SelectedEnsemble' + '-confs.mol2'))
            shutil.copyfile(newpath,ensemble_file)
            occ_file = os.path.join(save_dir, 'SelectedEnsemble-occ')
            total_occ = 0.0
            for rnum in rows:
                item = form.xgen_table.item(rnum,0)
                my_occ = float(item.text())
                total_occ = total_occ + my_occ
            f = open(occ_file,'w')
            for rnum in rows:
                item = form.xgen_table.item(rnum,0)
                my_occ = float(item.text())/total_occ
                f.write(format(my_occ,'.4f')+'\n')
            f.close

            sffuncs.maybe_load_sticks(newpath, 'SelectedEnsemble', 0.1, -1)    
            sffuncs.color_carbons('pink','SelectedEnsemble')
            cmd.hide('everything','(h. and (e. c extend 1))')
            cmd.disable('Ensemble')

            shutil.rmtree(tempdir)
                
    ################################################################
    
    def handle_exception(e, stdout):
        if isinstance(e, SilentAbort):
            return
        msg = str(e) or 'unknown error'
        msgbox = QMessageBox(QMessageBox.Critical, 'Error',
                             msg, QMessageBox.Close, form._dialog)
        if stdout.strip():
            msgbox.setDetailedText(stdout)
        msgbox.exec_()

##################################################################################
#  Use the Obtain button for aborting processes
#  Define running/not here
##################################################################################

    def set_running_abort():
        sfglob.abort_status = False
        form.tabWidget.setEnabled(False)
        form.button_register.clicked.disconnect()
        form.button_register.clicked.connect(abort)
        form.button_register.setText('Abort')
        form.button_register.setStyleSheet("background-color : red")

    def set_running_done():
        form.button_register.setText('Obtain SurFlex/xGen')
        form.button_register.clicked.disconnect()
        form.button_register.clicked.connect(surflex_register)
        form.button_register.setEnabled(True)
        form.button_register.setStyleSheet("background-color : light gray")
        form.tabWidget.setEnabled(True)
        
    # "Abort" button callback
    def abort():
        QMessageBox.about(form._dialog, "Aborting...", "Trying to abort the xGen process")
        sfglob.abort_status = True

        while form._proclist:
            p = form._proclist.pop()
            try:
                p.terminate()
                p.returncode = -15  # SIGTERM
            except OSError as e:
                print(e)

        form.button_register.clicked.disconnect()
        form.button_register.clicked.connect(surflex_register)
        form.button_register.setText('Obtain SurFlex/xGen')
        form.button_register.setEnabled(True)


##################################################################################
#   Hook up process fork button events to the sf_general machinery
##################################################################################
    @form.xgen_run_dbox.clicked.connect
    def _():
        sfglob.command = 'dbox'
        r_sfgeneral_pre()

    @form.xgen_run_build.clicked.connect
    def _():
        sfglob.command = 'build'
        r_sfgeneral_pre()

    @form.xgen_run_refine.clicked.connect
    def _():
        sfglob.command = 'refine'
        r_sfgeneral_pre()

    @form.xgens_run.clicked.connect
    def _():
        sfglob.command = 'ensemble'
        r_sfgeneral_pre()

    @form.xgeval_run.clicked.connect
    def _():
        sfglob.command = 'eval'
        r_sfgeneral_pre()


##################################################################################
#   Hook up interface-only functions (here, just xgen_select_ensemble)
##################################################################################
    form.xgen_select_ensemble.clicked.connect(xgen_select_ensemble)


##################################################################################
#   Hook up buttons that prompt user input
##################################################################################
    @form.xgen_outdir_browse.clicked.connect
    def _():
        my_dir = getExistingDirectory(None,
                                      "Select the xGen project folder",
                                      form.xgen_outdir.text(),
                                      QtWidgets.QFileDialog.ShowDirsOnly)
        form.xgen_outdir.setText(my_dir)

    @form.xgen_pdb_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='Protein (*.pdb);;All Files (*)')[0]
        if fnames:
            form.xgen_pdb.setText(homogenize_file(form,fnames[0]))

    @form.xgen_map_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='MTZ (*.mtz);;DSN6 (*.dsn6)')[0]
        if fnames:
            form.xgen_map.setText(homogenize_file(form,fnames[0]))

    @form.xgen_dbox_scope_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='MOL2 (*.mol2);;MDL (*.mol *.sdf *.sd)')[0]
        if fnames:
            form.xgen_dbox_scope.setText(homogenize_file(form,fnames[0]))
            form.xgen_refine_lig.setText(homogenize_file(form,fnames[0]))

    @form.xgen_refine_lig_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='MOL2 (*.mol2);;MDL (*.mol *.sdf *.sd)')[0]
        if fnames:
            form.xgen_refine_lig.setText(homogenize_file(form,fnames[0]))

    @form.xgen_constraints_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='XG Constraint File (*-ligfit)')[0]
        if fnames:
            form.xgen_constraints.setText(homogenize_file(form,fnames[0]))
            
    @form.xgen_protein_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='MOL2 (*.mol2);;PDB (*.pdb)')[0]
        if fnames:
            form.xgen_protein.setText(homogenize_file(form,fnames[0]))

    @form.orig_eval_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='MOL2 (*.mol2);;PDB (*.pdb)')[0]
        if fnames:
            form.orig_eval.setText(homogenize_file(form,fnames[0]))

    @form.orig_eval_occ_browse.clicked.connect
    def _():
        fnames = getOpenFileNames(None, filter='Occupancy file (*occ*)')[0]
        if fnames:
            form.orig_eval_occ.setText(homogenize_file(form,fnames[0]))

    @form.xgens_diverse.stateChanged.connect
    def _():
        if len(form.sf_outprefix.text()) > 0:
            if form.xgens_diverse.isChecked():
                form.xgens_input.setText((form.sf_outprefix.text() + '-trios.mol2'))
            else:
                form.xgens_input.setText((form.sf_outprefix.text() + '-densitybiased.mol2'))

    # "Register" opens a web browser
    @form.button_register.clicked.connect
    def _():
        import webbrowser
        webbrowser.open("https://www.biopharmics.com/downloads/")

    @form.browse_xgen_def_file.clicked.connect
    def _():
        from pymol import cmd
        fnames = getOpenFileNames(
            None, filter='xGen Definition File (*.xgen)')[0]
        if fnames:
            # Need to move to this directory
            head, tail = os.path.split(fnames[0])
            form.xgen_outdir.setText(os.path.realpath(head))
            cmd.cd(dir=os.path.realpath(head))
            form.xgen_def_file.setText(tail)
            read_xgenfile(fnames[0],form)

    form._dialog.show()
    form._dialog.resize(500, 600)

def __init_plugin__(app=None):
    from pymol.plugins import addmenuitemqt as addmenuitem
    addmenuitem('SurFlex: xGen Ligand Refinement', dialog)
