'''
Surflex xGen Refinement wrapper

(c) BioPharmics LLC

'''

from __future__ import print_function

import os
import types
import shutil
import sys
from . import sfglob

from pymol import cmd, CmdException

def maybe_load_sticks(inpath,obj_name,stick_radius,state):
    # Always delete the existing object
    if sele_exists(obj_name):
        cmd.delete(obj_name)
        
    # Check if path exisits
    if not os.path.exists(inpath):
        return

    if os.path.isdir(inpath):
        return

    cmd.load(inpath, obj_name, zoom=0)
    cmd.show_as('sticks',obj_name)
    cmd.set('stick_radius', stick_radius, obj_name)
    cmd.set('state', state, obj_name)

def concat_files(filenames, outfile):
    # Open file in write mode 
    with open(outfile, 'w') as outfile: 
    
        # Iterate through list 
        for names in filenames: 
        
            # Open each file in read mode 
            with open(names) as infile: 
            
                # read the data from infile and 
                # write it in outfile
                outfile.write(infile.read()) 


def color_carbons(color,selection,quiet=1,_self=cmd):
    cmd=_self
    s = str(selection)
    if not sele_exists(s):
        return
    cmd.color("atomic","(("+s+") and not elem C)",quiet=quiet)
    cmd.color(color,"(elem C and ("+s+"))",quiet=quiet)    

def color_all(color,selection,quiet=1,_self=cmd):
    cmd=_self
    s = str(selection)
    if not sele_exists(s):
        return
    cmd.color(color,"(("+s+"))",quiet=quiet)    
        
def sele_exists(sele):
    sess = cmd.get_session()
    for i in sess["names"]:
        if type(i) is list:
            if sele==i[0]:
                return 1
    return 0

def find_surflex_exe():
    exe = cmd.exp_path('$HOME/bin/sf-xgen.exe')
    if not os.path.exists(exe):
        from distutils import spawn
        exe = spawn.find_executable('sf-xgen.exe')
    return exe

def validate_surflex_exe(exe):
    '''Get and validate surflex executable.
    Raise CmdException if not found or broken.'''
    import subprocess

    if exe:
        exe = cmd.exp_path(exe)
    else:
        exe = find_surflex_exe() or 'surflex'

    try:
        r = subprocess.call([exe, "--version"],
                stdin=subprocess.PIPE,  # Windows pythonw fix
                stdout=open(os.devnull, "w"), stderr=subprocess.STDOUT)
        if r < 0:
            raise CmdException("Broken executable: " + exe)
    except OSError as e:
        print(e)
        raise CmdException("Cannot execute: " + exe)

    return exe


def run_xgen_refine(assign=0, preserve=1, quiet=1, state=1, _proclist=None):
    '''
DESCRIPTION

    Runs xGen refinement search on specified input file using specified mode.

    '''
    import tempfile, shutil, glob, subprocess

    xg_lig = sfglob.xg_lig
    xg_string = sfglob.xg_string
    mode_name = sfglob.mode_name
    outprefix = sfglob.sf_outprefix
    save_dir = sfglob.save_dir
    options = sfglob.options
    exe = sfglob.tools_exe
    preserve = sfglob.check_preserve
    quiet = int(quiet)

    print('Inside run_xgen: ', xg_lig, xg_string, mode_name )

    # path to surflex executable
    exe = validate_surflex_exe(exe)

    # temporary directory
    tempdir = tempfile.mkdtemp()

    print('Inside run_xgen_refine, tempdir: ', tempdir )

    # filenames
    outfile = os.path.join(tempdir, outprefix)
    molfile = os.path.join(tempdir, 'mol.mol2')
    xgfile = os.path.join(tempdir, 'xg-ligfit')

    # Copy input ligand and xgen restraint file to temp dir
    shutil.copyfile(xg_lig,molfile)
    shutil.copyfile(xg_string,xgfile)

    try:
        # run xgen
        # Need to concat the options into the arg list
        if len(options) > 0:
            option_args = options.split()
            arg_list = [exe] + option_args + [mode_name, 'refine', xgfile, molfile, outprefix]
        else:
            arg_list = [exe, mode_name, 'refine', xgfile, molfile, outprefix]

        p = subprocess.Popen(arg_list, cwd=tempdir)

        if _proclist is not None:
            _proclist.append(p)
        
            r = p.wait()
        
            if r == -15: # SIGTERM
                raise CmdException('sf-xgen terminated')

            if r < 0:
                raise CmdException('sf-xgen failed with code ' + str(r))

            if r != 0: 
                shutil.rmtree(tempdir)
            
            if r == 0:
                print('  FINISHED SEARCH!',file=sys.stderr)
                # load output conformers
                sfdbfile = os.path.join(tempdir, (outprefix + '.sfdb'))
                p = subprocess.Popen([exe, 'expand', xgfile, sfdbfile, molfile, (outprefix + '-prof')], cwd=tempdir)
                r = p.wait()
                conf_name = 'Density-Biased'
                mol2file = os.path.join(tempdir, (outprefix + '-prof-best-density.mol2'))
                maybe_load_sticks(mol2file,conf_name,0.1,-1)
                color_carbons('orange','Density-Biased')
                
                conf_name = 'Blended'
                mol2file = os.path.join(tempdir, (outprefix + '-prof-best-center.mol2'))
                maybe_load_sticks(mol2file,conf_name,0.1,-1)
                color_carbons('slate','Blended')
                
                conf_name = 'Energy-Biased'
                mol2file = os.path.join(tempdir, (outprefix + '-prof-best-min.mol2'))
                maybe_load_sticks(mol2file,conf_name,0.1,-1)
                color_carbons('yellow','Energy-Biased')
                
                cmd.group('Trios', 'Density-Biased Blended Energy-Biased')
                cmd.show_as('sticks','Trios')
                
                cmd.hide('everything','(h. and (e. c extend 1))')
                
                # Copy SFDB file, trios, and log file to save_dir
                to_sfdbfile = os.path.join(save_dir, (outprefix + '.sfdb'))
                shutil.copyfile(sfdbfile,to_sfdbfile)
                
                filenames = [os.path.join(tempdir, (outprefix + '-prof-best-density.mol2')), 
                             os.path.join(tempdir, (outprefix + '-prof-best-center.mol2')),
                             os.path.join(tempdir, (outprefix + '-prof-best-min.mol2'))]
                to_fitfile = os.path.join(save_dir, (outprefix + '-trios.mol2'))
                concat_files(filenames,to_fitfile)
                
                logfile = os.path.join(tempdir, (outprefix + '-prof-log'))
                to_logfile = os.path.join(save_dir, (outprefix + '-trios-log'))
                shutil.copyfile(logfile,to_logfile)

                bestfile = os.path.join(tempdir, (outprefix + '-prof-best-density.mol2'))
                to_fitfile = os.path.join(save_dir, (outprefix + '-densitybiased.mol2'))
                shutil.copyfile(bestfile,to_fitfile)

                bestfile = os.path.join(tempdir, (outprefix + '-prof-best-center.mol2'))
                to_fitfile = os.path.join(save_dir, (outprefix + '-best-center.mol2'))
                shutil.copyfile(bestfile,to_fitfile)

                bestfile = os.path.join(tempdir, (outprefix + '-prof-best-density.mol2'))
                to_fitfile = os.path.join(save_dir, (outprefix + '-best-density.mol2'))
                shutil.copyfile(bestfile,to_fitfile)

                bestfile = os.path.join(tempdir, (outprefix + '-prof-best-min.mol2'))
                to_fitfile = os.path.join(save_dir, (outprefix + '-best-min.mol2'))
                shutil.copyfile(bestfile,to_fitfile)
                
    except OSError as e:
        print(e)
        raise CmdException('Cannot execute "%s"' % (exe))
    finally:
        if not preserve:
            shutil.rmtree(tempdir)
        elif not quiet:
            print(' Notice: not deleting %s' % (tempdir))


def run_build_dbox(assign=0, preserve=1, quiet=1, state=1, _proclist=None):
    '''
DESCRIPTION

    Runs xGen density box building process, including extracting real-space density

    '''
    import tempfile, shutil, glob, subprocess

    protein = sfglob.xgen_protein
    map_file = sfglob.dbox_map
    scope_ligand = sfglob.dbox_scope
    outprefix = sfglob.xgen_outprefix
    resolution = sfglob.xgen_res
    xg_carve = sfglob.xgen_dbox_carve
    xg_lig = sfglob.xg_lig
    xg_string = sfglob.xg_string
    mode_name = sfglob.mode_name
    save_dir = sfglob.save_dir
    options = sfglob.options
    exe = sfglob.tools_exe
    preserve = sfglob.check_preserve
    quiet = int(quiet)

    print('Inside run_build_dbox: ', protein, map_file, scope_ligand )

    cmd.disable()

    maybe_load_sticks(protein, 'xg_protein', 0.10,0)
    color_carbons('gray40','xg_protein')
    if not sele_exists('phases.2fofc'):
        cmd.load(map_file, 'phases', quiet=1)
    maybe_load_sticks(scope_ligand, 'xg_scope', 0.15, -1)
    color_carbons('lightpink','xg_scope')
    cmd.hide('everything','(h. and (e. c extend 1))')
    if sele_exists('map'):
        cmd.delete('map')

    # Now we need to get a temp directory to dump the mesh...
    tempdir = tempfile.mkdtemp()
    cmd.zoom('xg_scope')
    dbox_outfile = os.path.join(tempdir, 'raw_density_box.pdb')
    dfile = open(dbox_outfile,'w');
    dumpfile = os.path.join(tempdir, 'mesh.txt')
    
    for i in range (50,-51,-1):
        mesh_level = i/10.0;
        cmd.isomesh('map','phases.2fofc',level=mesh_level,selection='xg_scope',state=1,carve=xg_carve)
        cmd.dump(dumpfile,'map')
        f = open(dumpfile,'r')
        for line in f:
            if len(line) > 1:
                coords = line.split()
                x = float(coords[0])
                y = float(coords[1])
                z = float(coords[2])
                resnum = 5000-i;
                pdbstring = "HETATM {:4d}  C   UNK U   1    {:8.3f}{:8.3f}{:8.3f}  1.00 {:3.2f}            C\n"
                new_line = pdbstring.format(resnum,x,y,z,mesh_level)
                dfile.write(new_line)
        f.close()       
    dfile.close()

    # Just for pretty: Let's contour at 1 sigma
    cmd.isomesh('map','phases.2fofc',level=1.0,selection='xg_scope',state=1,carve=xg_carve)
    cmd.set('mesh_color','gray60')
    
    # path to surflex executable
    exe = validate_surflex_exe(exe)

    # So now we have the density box. We can finally run xgen_dbox
    #    sf-xgen.exe dbox raw_density_box.pdb lig-orig.mol2 pdb-res density
    # This will sample and smooth the real-space density on a 0.25A grid and
    # will build contour data files for fitting the idealized density function

    raw_path = os.path.join(tempdir, 'raw_density_box.pdb')
    res_path = os.path.join(tempdir, 'pdb_res')
    rfile = open(res_path,'w')
    rfile.write(format(resolution,".3f") + '\n')
    rfile.close()

    # Need to concat the options into the arg list
    if len(options) > 0:
        option_args = options.split()
        arg_list = [exe] + option_args + ['-xg_dcarve', format(xg_carve,'.2f'), 'dbox', raw_path, scope_ligand, res_path, outprefix]
    else:
        arg_list = [exe, '-xg_dcarve', format(xg_carve,'.2f'), 'dbox', raw_path, scope_ligand, res_path, outprefix]

    try:
        p = subprocess.Popen(arg_list, cwd=save_dir)
        if _proclist is not None:
            _proclist.append(p)
        
            r = p.wait()
        
            if r == -15: # SIGTERM
                raise CmdException('sf-xgen terminated')

            if r < 0:
                raise CmdException('sf-xgen failed with code ' + str(r))

            if r != 0: 
                shutil.rmtree(tempdir)
            
            if r == 0:
                # Now load the density contour, show as sticks, radius 0.03, color by partial charge
                contour_path = os.path.join(save_dir, (outprefix + '_contour.mol2'))
                maybe_load_sticks(contour_path, 'Raw_Contour','0.03','0')
                cmd.spectrum('pc', 'rainbow', 'Raw_Contour')
                cmd.hide('everything','(h. and (e. c extend 1))')
                if sele_exists('map'): cmd.disable('map')
                

    except OSError as e:
        print(e)
        raise CmdException('Cannot execute "%s"' % (exe))
    finally:
        if not preserve:
            shutil.rmtree(tempdir)
        elif not quiet:
            print(' Notice: not deleting %s' % (tempdir))


def run_build_xgbuild(assign=0, focus='', preserve=1, quiet=1, state=1,_proclist=None):
    '''
DESCRIPTION

    Runs xGen idealized density building process

    '''
    import tempfile, shutil, glob, subprocess

    protein = sfglob.xgen_protein
    scope_ligand = sfglob.dbox_scope
    outprefix = sfglob.build_outprefix
    dprefix = sfglob.dprefix
    xg_carve = sfglob.xgen_build_carve
    save_dir = sfglob.save_dir
    options = sfglob.options
    exe = sfglob.tools_exe
    preserve = sfglob.check_preserve
    xg_contour = sfglob.xg_build_contour
    quiet = int(quiet)

    if sele_exists('Ensemble'): cmd.delete('Ensemble')
    if sele_exists('Ensemble-Iso10'): cmd.delete('Ensemble-Iso10')
    if sele_exists('XGEval_Contour1.0'): cmd.delete('XGEval_Contour1.0')
    if sele_exists('OrigEval_Contour1.0'): cmd.delete('OrigEval_Contour1.0')

    if not sele_exists('xg_protein'):
        cmd.load(protein, 'xg_protein', quiet=1)
        color_carbons('gray40','xg_protein')
    maybe_load_sticks(scope_ligand, 'xg_scope', 0.15, -1)
    color_carbons('lightpink','xg_scope')
    cmd.zoom('xg_scope')

    cmd.hide('everything','(h. and (e. c extend 1))')

    # Need to call: sf-xgen.exe +xg_contour build lig-orig.mol2 protein.mol2 density xg
    # Do not need a temporary directory. We will do this in the save_dir
    # path to surflex executable
    exe = validate_surflex_exe(exe)

    if xg_contour:
        contour_arg = ['+xg_contour']
    else:
        contour_arg = ['-xg_contour']


    # Need to concat the options into the arg list
    if len(options) > 0:
        option_args = options.split()
        arg_list = [exe] + option_args + contour_arg + ['-xg_carve', format(xg_carve,'.2f'), 'build', scope_ligand, protein, dprefix, outprefix]
    else:
        arg_list = [exe] + contour_arg + ['-xg_carve', format(xg_carve,'.2f'), 'build', scope_ligand, protein, dprefix, outprefix]

    try:
        p = subprocess.Popen(arg_list, cwd=save_dir)
        if _proclist is not None:
            _proclist.append(p)
        
            r = p.wait()
        
            if r == -15: # SIGTERM
                raise CmdException('sf-xgen terminated')

            if r < 0:
                raise CmdException('sf-xgen failed with code ' + str(r))

            if r == 0:
                # Now load the idealized density contours, show as sticks, radius 0.03, color by partial charge
                contour_path = os.path.join(save_dir, (outprefix + '-fit00-iso.mol2'))
                maybe_load_sticks(contour_path, 'Ideal00_Contour','0.03','0')
                contour_path = os.path.join(save_dir, (outprefix + '-fit01-iso.mol2'))
                maybe_load_sticks(contour_path, 'Ideal01_Contour','0.03','0')
                contour_path = os.path.join(save_dir, (outprefix + '-fit02-iso.mol2'))
                maybe_load_sticks(contour_path, 'Ideal02_Contour','0.03','0')
                contour_path = os.path.join(save_dir, (outprefix + '-fit03-iso.mol2'))
                maybe_load_sticks(contour_path, 'Ideal03_Contour','0.03','0')
                contour_path = os.path.join(save_dir, (outprefix + '-fit04-iso.mol2'))
                maybe_load_sticks(contour_path, 'Ideal04_Contour','0.03','0')
                
                cmd.group('Ideal_Contours', 'Ideal00_Contour Ideal01_Contour Ideal02_Contour Ideal03_Contour Ideal04_Contour')
                cmd.show_as('sticks','Ideal_Contours')
                cmd.set('stick_radius', '0.03', 'Ideal_Contours')
                cmd.spectrum('pc', 'rainbow', 'Ideal_Contours')
                cmd.enable('Exptl_Contours')
                
                loadfile = os.path.join(save_dir, (outprefix + '-exptl00-iso.mol2'))
                maybe_load_sticks(loadfile,'Exptl00_Contour','0.03','0')
                loadfile = os.path.join(save_dir, (outprefix + '-exptl01-iso.mol2'))
                maybe_load_sticks(loadfile,'Exptl01_Contour','0.03','0')
                loadfile = os.path.join(save_dir, (outprefix + '-exptl02-iso.mol2'))
                maybe_load_sticks(loadfile,'Exptl02_Contour','0.03','0')
                loadfile = os.path.join(save_dir, (outprefix + '-exptl03-iso.mol2'))
                maybe_load_sticks(loadfile,'Exptl03_Contour','0.03','0')
                loadfile = os.path.join(save_dir, (outprefix + '-exptl04-iso.mol2'))
                maybe_load_sticks(loadfile,'Exptl04_Contour','0.03','0')
                if sele_exists('Exptl00_Contour'):
                    cmd.group('Exptl_Contours', 'Exptl00_Contour Exptl01_Contour Exptl02_Contour Exptl03_Contour Exptl04_Contour')
                    cmd.show_as('sticks','Exptl_Contours')
                    cmd.set('stick_radius', '0.03', 'Exptl_Contours')
                    cmd.spectrum('pc', 'rainbow', 'Exptl_Contours')
                    cmd.disable('Exptl_Contours')
                if sele_exists('Raw_Contour'): cmd.disable('Raw_Contour')

    except OSError as e:
        print(e)
        raise CmdException('Cannot execute "%s"' % (exe))


def run_xgensemble(assign=0, focus='', preserve=1, quiet=1, state=1,_proclist=None):
    '''
DESCRIPTION

    Runs xGen ensemble building process

    '''
    import tempfile, shutil, glob, subprocess

    xgstring = sfglob.xgen_constraints
    input_pool = sfglob.input_pool
    xgen_protein = sfglob.xgen_protein
    refine_ligand = sfglob.xg_lig
    dprefix = sfglob.dprefix
    outprefix = sfglob.xgens_outprefix
    xg_contour = sfglob.xgens_contour
    xg_diverse = sfglob.xgens_diverse
    quiet = int(quiet)
    exe = sfglob.tools_exe
    options = sfglob.options
    save_dir = sfglob.save_dir

    # path to surflex executable
    exe = validate_surflex_exe(exe)

    maybe_load_sticks(xgen_protein, 'xg_protein', '0.10','-1')
    color_carbons('gray40','xg_protein')
    maybe_load_sticks(refine_ligand, 'xg_scope', '0.11','-1')
    if sele_exists('xg_scope'):
        cmd.zoom('xg_scope')
        color_carbons('lightpink','xg_scope')

    cmd.hide('everything','(h. and (e. c extend 1))')

    if sele_exists('Ensemble'): cmd.delete('Ensemble')

    if xg_contour:
        contour_arg = ['+xg_contour']
    else:
        contour_arg = ['-xg_contour']

    if xg_diverse:
        div_arg = ['-pdiverse']
    else:
        div_arg = ['-pstrict']

    # Need to concat the options into the arg list
    if len(options) > 0:
        option_args = options.split()
        arg_list = [exe] + option_args + contour_arg + div_arg + ['ensemble', xgstring, dprefix, xgen_protein, refine_ligand, input_pool, outprefix]
    else:
        arg_list = [exe] + contour_arg + div_arg + ['ensemble', xgstring, dprefix, xgen_protein, refine_ligand, input_pool, outprefix]

    # Run it
    try:
        p = subprocess.Popen(arg_list, cwd=save_dir)
        if _proclist is not None:
            _proclist.append(p)
        
            r = p.wait()
        
            if r == -15: # SIGTERM
                raise CmdException('sf-xgen terminated')

            if r < 0:
                raise CmdException('sf-xgen failed with code ' + str(r))

            if r == 0:

                if sele_exists('Trios'): cmd.disable('Trios')

                # Now load the ensemble
                ensemble_path = os.path.join(save_dir, (outprefix + '-confs.mol2'))
                maybe_load_sticks(ensemble_path,'Ensemble','0.10','-1')
                color_carbons('brightorange','Ensemble')

                # Now load the idealized density contours, show as sticks, radius 0.03
                contour_path = os.path.join(save_dir, (outprefix + '-iso.mol2'))
                maybe_load_sticks(contour_path,'Ensemble-Iso10','0.03','-1')
                color_all('brightorange','Ensemble-Iso10')
                cmd.hide('everything','(h. and (e. c extend 1))')

    except OSError as e:
        print(e)
        raise CmdException('Cannot execute "%s"' % (exe))


def run_xgeval(assign=0, focus='', preserve=1, quiet=1, state=1, _proclist=None):
    '''
    Runs xGen eval process

    '''
    import tempfile, subprocess

    ensemble_prefix = sfglob.xgens_outprefix
    ensemble_string = sfglob.xgens_mol2
    occ_string = sfglob.xgens_occ
    xgen_protein = sfglob.xgen_protein
    protein_pdb = sfglob.xgen_protein_pdb
    refine_ligand = sfglob.xg_lig
    dprefix = sfglob.dprefix
    xg_contour = sfglob.xgens_contour
    orig_eval_lig = sfglob.origeval_mol2
    orig_eval_occ = sfglob.origeval_occ
    quiet = int(quiet)
    exe = sfglob.tools_exe
    options = sfglob.options
    save_dir = sfglob.save_dir
    
    # Load the pdb protein map
    cmd.disable()
    cmd.load(protein_pdb, 'Protein_BFactors', quiet=1)    
    cmd.spectrum('b', 'rainbow', 'Protein_BFactors')
    if len(orig_eval_lig) > 0:
        maybe_load_sticks(orig_eval_lig,'Orig_Lig','0.1','-1')
    else:
        maybe_load_sticks(refine_ligand,'Orig_Lig','0.1','-1')
                
    color_carbons('cyan','Orig_Lig')
    cmd.zoom('Orig_Lig')
    if sfglob.xgens_mol2 == 'SelectedEnsemble-confs.mol2':
        cmd.enable('SelectedEnsemble')
    else:
        cmd.enable('Ensemble')

    if not sele_exists('map'):
        mapfile = sfglob.mapfile
        cmd.load(mapfile, 'phases', quiet=1)
    cmd.isomesh('map','phases.2fofc',level=1.0,selection='Orig_Lig',state=1,carve=1.6)
    cmd.hide('everything','(h. and (e. c extend 1))')
    cmd.set('mesh_color','gray60')
    cmd.enable('map')
    cmd.zoom('Orig_Lig')

    if len(orig_eval_lig) > 0:
        eval_ligand = orig_eval_lig
    else:
        eval_ligand = refine_ligand    
    if len(orig_eval_occ) > 0:
        eval_occ = orig_eval_occ
    else:
        eval_occ = 'equal_occ'

    # Need to make the xgeval scope ligand
    filenames = [os.path.join(save_dir,eval_ligand), os.path.join(save_dir,ensemble_string)]
    eval_scope = os.path.join(save_dir, (ensemble_prefix + '-eval-scope.mol2'))
    concat_files(filenames,eval_scope)

    exe = validate_surflex_exe(exe)

    eval_prefix = (ensemble_prefix + '-eval')

    if xg_contour:
        contour_arg = ['+xg_contour']
    else:
        contour_arg = ['-xg_contour']

    # Need to concat the options into the arg list
    if len(options) > 0:
        option_args = options.split()
        arg_list = [exe] + option_args + contour_arg + ['eval',occ_string, ensemble_string, xgen_protein, eval_scope, protein_pdb, dprefix, eval_prefix]
    else:
        arg_list = [exe] + contour_arg + ['eval',occ_string, ensemble_string, xgen_protein, eval_scope, protein_pdb, dprefix, eval_prefix]

    # Run it
    try:
        p = subprocess.Popen(arg_list, cwd=save_dir)
        if _proclist is not None:
            _proclist.append(p)
        
            r = p.wait()
        
            if r == -15: # SIGTERM
                raise CmdException('sf-xgen terminated')

            if r < 0:
                raise CmdException('sf-xgen failed with code ' + str(r))

            if r == 0:
                # Now load the eval iso
                if xg_contour:
                    contour_path = os.path.join(save_dir, (eval_prefix + '-iso10.mol2'))
                    maybe_load_sticks(contour_path,'XGEval_Contour1.0','0.03','0')
                    color_all('brightorange','XGEval_Contour1.0')

    except OSError as e:
        print(e)
        raise CmdException('Cannot execute "%s"' % (exe))

    pdbeval_prefix = (ensemble_prefix + '-pdbeval')
    ptile_string = (eval_prefix + '-ptile')

    # Need to concat the options into the arg list
    if len(options) > 0:
        option_args = options.split()
        arg_list = [exe] + option_args + contour_arg + ['-xg_ptile',ptile_string, 'eval',eval_occ, eval_ligand, xgen_protein, eval_scope, protein_pdb, dprefix, pdbeval_prefix]
    else:
        arg_list = [exe] + contour_arg + ['-xg_ptile',ptile_string, 'eval',eval_occ, eval_ligand, xgen_protein, eval_scope, protein_pdb, dprefix, pdbeval_prefix]

#    p = subprocess.Popen([exe, '-xg_ptile',ptile_string, '+xg_contour','eval',eval_occ, eval_ligand, xgen_protein, eval_scope, protein_pdb, dprefix, pdbeval_prefix], cwd=save_dir)
    # Run it
    try:
        p = subprocess.Popen(arg_list, cwd=save_dir)
        if _proclist is not None:
            _proclist.append(p)
        
            r = p.wait()
        
            if r == -15: # SIGTERM
                raise CmdException('sf-xgen terminated')

            if r < 0:
                raise CmdException('sf-xgen failed with code ' + str(r))

            if r == 0:
                # Now load the eval iso
                if xg_contour:
                    contour_path = os.path.join(save_dir, (pdbeval_prefix + '-iso10.mol2'))
                    maybe_load_sticks(contour_path,'OrigEval_Contour1.0','0.03','0')
                    color_all('cyan','OrigEval_Contour1.0')

    except OSError as e:
        print(e)
        raise CmdException('Cannot execute "%s"' % (exe))
